﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

using Infragistics.Controls.Charts;
using System.Windows;

namespace WpfInfragistics18Experiments
{ 
     
    /// <summary>
    /// Represents base class for custom scalers
    /// </summary>
    public abstract class CustomNumericScaler : NumericScaler
    {             
        public double DataMinimumValue
        {
            get { return (double)GetValue(DataMinimumValueProperty); }
            set { SetValue(DataMinimumValueProperty, value); }
        }
         
        public static readonly DependencyProperty DataMinimumValueProperty =
            DependencyProperty.Register("DataMinimumValue", typeof(double), 
            typeof(CustomNumericScaler), new PropertyMetadata(0.0));
        
        public double DataMaximumValue
        {
            get { return (double)GetValue(DataMaximumValueProperty); }
            set { SetValue(DataMaximumValueProperty, value); }
        }
        
        public static readonly DependencyProperty DataMaximumValueProperty =
            DependencyProperty.Register("DataMaximumValue", 
            typeof(double), typeof(CustomNumericScaler), new PropertyMetadata(10.0));

        public double DataRange { get { return this.DataMaximumValue - this.DataMinimumValue; } }

        public double ScaleMinimumValue
        {
            get { return (double)GetValue(ScaleMinimumValueProperty); }
            set { SetValue(ScaleMinimumValueProperty, value); }
        }
         
        public static readonly DependencyProperty ScaleMinimumValueProperty =
            DependencyProperty.Register("ScaleMinimumValue", typeof(double), 
            typeof(CustomNumericScaler), new PropertyMetadata(0.0));
        
        public double ScaleMaximumValue
        {
            get { return (double)GetValue(ScaleMaximumValueProperty); }
            set { SetValue(ScaleMaximumValueProperty, value); }
        }
        
        public static readonly DependencyProperty ScaleMaximumValueProperty =
            DependencyProperty.Register("ScaleMaximumValue", 
            typeof(double), typeof(CustomNumericScaler), new PropertyMetadata(10.0));

        public double ScaleRange { get { return this.ScaleMaximumValue - this.ScaleMinimumValue; } }
        /// <summary>
        /// Calculates the actual minimum and actual maximum values for the axis scale
        /// </summary>
        public override void CalculateRange(NumericAxisBase target, double minimumValue, double maximumValue,
                                            out double actualMinimumValue,
                                            out double actualMaximumValue)
        { 
            actualMinimumValue = this.ScaleMinimumValue;
            actualMaximumValue = this.ScaleMaximumValue;
        }

        #region Unchangeabgle functions for converting to and from window coordinates
        public double ConvertFromWindowY(double y, ScalerParams p)
        {
            return p.ViewportRect.Top + p.ViewportRect.Height * (y - p.WindowRect.Top) / p.WindowRect.Height;
        }
        public double ConvertToWindowY(double y, ScalerParams p)
        {
            return p.WindowRect.Top + p.WindowRect.Height * (y - p.ViewportRect.Top) / p.ViewportRect.Height;
        }
        public double ConvertFromWindowX(double x, ScalerParams p)
        {
            return p.ViewportRect.Left + p.ViewportRect.Width * (x - p.WindowRect.Left) / p.WindowRect.Width;
        }
        public double ConvertToWindowX(double x, ScalerParams p)
        {
            return p.WindowRect.Left + p.WindowRect.Width * (x - p.ViewportRect.Left) / p.ViewportRect.Width;
        } 
        #endregion
    }

    /// <summary>
    /// Represents custom scaler for  numeric axis
    /// </summary>
    public class CustomHorizontalScaler : CustomNumericScaler
    {  
        /// <summary>
        /// Returns the display (pixel) position corresponding with a numeric value on the axis
        /// </summary>
        public override double GetScaledValue(double unscaledValue, ScalerParams p)
        {
            // var acutal = ScaleMin + v * (ScaleMax - ScaleMin) / ScaleFactor;

            //double scaledValue = ((unscaledValue * this.DataRange) * this.DataRange) / this.ScaleRange;

            // double scaledValue = ((unscaledValue - this.ScaleMinimumValue) / this.DataRange) * this.ScaleRange;
            double scaledValue = ((unscaledValue - this.ScaleMinimumValue) / this.ScaleRange); // * this.DataRange;
            //  double scaledValue = ((unscaledValue * this.DataRange - this.ScaleMinimumValue) / this.ScaleRange);
            if (p.IsInverted)
            {
                scaledValue = 1.0 - scaledValue;
            }
            var ret = ConvertFromWindowX(scaledValue, p);
            return ret;
        }
        /// <summary>
        /// Returns the numeric value corresponding to the display or "pixel" position on the axis
        /// </summary>
        public override double GetUnscaledValue(double scaledValue, ScalerParams p)
        {
            double unscaledValue = ConvertToWindowX(scaledValue, p);
            if (p.IsInverted)
            {
                unscaledValue = 1.0 - unscaledValue;
            }
            // var acutal = ScaleMin + v * (ScaleMax - ScaleMin) / ScaleFactor;            
            // var dataValue = this.ScaleMinimumValue + (unscaledValue * this.ScaleRange) / this.DataRange; 
            var dataValue = this.ScaleMinimumValue + unscaledValue * this.ScaleRange / this.DataRange;             
            return dataValue;
        }




    }
}