﻿using System;
using System.Collections.Generic;
using System.Collections.Specialized;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;
using Infragistics.Controls.Charts;

namespace WpfInfragistics18Experiments
{ 
    public partial class Window2 : Window
    {
        private PdfViewModel VM;
        public Window2()
        {
            //xConverter = new AxisLabelConverter();
            //Resources["XAxisLabelConverter"] = xConverter;

            InitializeComponent();
        }
        
        private void AddXHistogram(DataPointCollection dataPointCollection, Color color)
        {
            var series = new ScatterLineSeries();
            series.ItemsSource = dataPointCollection;
            series.XMemberPath = nameof(DataPoint.X);
            series.YMemberPath = nameof(DataPoint.Z);
            series.Title = dataPointCollection.Name;
            series.XAxis = xHistXAxis;
            series.YAxis = xHistYAxis;
            series.MarkerType = MarkerType.Circle;
            series.Legend = legend;
            series.ShowDefaultTooltip = true;
            series.MarkerBrush = new SolidColorBrush(color);
            series.Brush = new SolidColorBrush(color);
            xHist.Series.Add(series);
        }

        private void AddYHistogram(DataPointCollection dataPointCollection, Color color)
        {
            var series = new ScatterLineSeries();
            series.ItemsSource = dataPointCollection;
            series.XMemberPath = nameof(DataPoint.Z);
            series.YMemberPath = nameof(DataPoint.Y);
            series.Title = dataPointCollection.Name;
            series.XAxis = yHistXAxis;
            series.YAxis = yHistYAxis;
            series.MarkerType = MarkerType.Circle;
            //series.UseLightweightMarkers = true;
            //series.MarkerStyle = new Style();
            //series.MarkerStyle.Setters.Add(new Setter(WidthProperty, 1));
            //series.MarkerStyle.Setters.Add(new Setter(HeightProperty, 1));
            series.Legend = legend;
            series.ShowDefaultTooltip = true;
            series.MarkerBrush = new SolidColorBrush(color);
            series.Brush = new SolidColorBrush(color);
            yHist.Series.Add(series);
        }

        private void AddContour(DataPointCollection dataPointCollection, XamDataChart dataChart, Color color)
        {
            var series = new ScatterContourSeries();
            series.ItemsSource = dataPointCollection;
            series.XMemberPath = nameof(DataPoint.X);
            series.YMemberPath= nameof(DataPoint.Y);
            series.ValueMemberPath = nameof(DataPoint.Z);
            series.XAxis = dataChart.Axes[0] as NumericXAxis;
            series.YAxis = dataChart.Axes[1] as NumericYAxis;
            series.ShowDefaultTooltip = true;
            series.Legend = legend;
            series.Brush = new SolidColorBrush(color);
            series.ValueResolver = new LinearContourValueResolver() {ValueCount = 12};
            //series.FillScale = new ValueBrushScale { MinimumValue = 0,  MaximumValue = 1 };
            dataChart.Series.Add(series);
        }
        
        private AxisLabelConverter xConverter;
        private AxisLabelConverter yConverter;

        private void Window_Loaded(object sender, RoutedEventArgs e)
        {
            //VM = DataContext as PdfViewModel;
            xConverter = this.Resources["xAxisLabelConverter"] as AxisLabelConverter;
            yConverter = this.Resources["yAxisLabelConverter"] as AxisLabelConverter;

            VM = this.Resources["VM"] as PdfViewModel;
            if (VM != null)
            {
                VM.PdfCollection.CollectionChanged += PointCollectionsOnCollectionChanged; 
                VM.LoadData(); 

                // setting scale parameters on the X converter
                xConverter.ScaleMin = VM.XRange.Min;
                xConverter.ScaleMax = VM.XRange.Max; 
                xConverter.ScaleFactor = VM.XBins;
                // passing axis functinality to the X converter
                xConverter.UseAbbreviation = this.xAxis.AbbreviateLargeNumbers;
                xConverter.UseRounding = true;
                                
                // setting scale parameters on the Y converter
                yConverter.ScaleMin = VM.YRange.Min;
                yConverter.ScaleMax = VM.YRange.Max; 
                yConverter.ScaleFactor = VM.YBins;
                // passing axis functinality to the Y converter
                yConverter.UseAbbreviation = this.xAxis.AbbreviateLargeNumbers;
                yConverter.UseRounding = false;
                  
                foreach (var data in VM.PdfCollection)
                { 
                    GenerateCharts(data);
                } 

            }
        }

        private void PointCollectionsOnCollectionChanged(object sender, NotifyCollectionChangedEventArgs args)
        {
            if (args.Action == NotifyCollectionChangedAction.Add)
            {
                foreach (var newItem in args.NewItems)
                {
                    var pointCollection = newItem as PdfPlotData;
                    GenerateCharts(pointCollection);
                }
            }
        }

        private void GenerateCharts(PdfPlotData data)
        {
            if (data == null) return;

            AddContour(data.Points, contourChart, data.Color);
            //AddContour(data.Points, contourChart2, data.Color);
            AddXHistogram(data.XHistogram, data.Color);
            AddYHistogram(data.YHistogram, data.Color);
        }


    }
}
